/*
 * gtkmonop - the gtk+ monopoly simulator
 * Copyright (C) 2001 Erik Bourget
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <string.h>

#include "stub.h"
#include "structs.h"
#include "misc_functions.h"
#include "gdk_functions.h"

static GdkPixmap *boardpix;
static GdkPixmap *numbers;
static GdkPixmap *flags;
static GdkPixmap *mflags;
static GdkPixmap *house;
static GdkPixmap *deeds;

/** Styles for tab labels */
GtkStyle *active_text_style;
GtkStyle *inactive_text_style;

static GtkWidget *board;
extern player *players;
static GdkBitmap *bmap, *nmap, *fmap, *mmap, *hmap, *dmap;

static int base_house_coords[] = {
	/* coordinates to start drawing houses
	   on properties */
	400, 400, /* Go */
	354, 393,
	317, 393,
	280, 393,
	243, 393,
	207, 393, /* RR */
	170, 393,
	133, 393,
	96, 393,
	59, 393,
	21, 393, /* Jail */
	44, 355,
	44, 318,
	44, 281,
	44, 244,
	44, 207, /* RR */
	44, 170,
	44, 133,
	44, 96,
	44, 59,
	20, 24, /* Free Parking */
	61, 44,
	97, 44,
	134, 44,
	171, 44,
	208, 44, /* RR */
	245, 44,
	282, 44,
	318, 44,
	355, 44,
	396, 20, /* ->jail */
	392, 60,
	392, 97,
	392, 134,
	392, 171,
	392, 208, /* RR */
	392, 245,
	392, 282,
	392, 319,
	392, 356
};



static int base_coords[] = {  
	/* coordinates to start drawing little
	   icons on the board */

	400, 400, /* Go */
	355, 400,
	318, 400,
	281, 400,
	244, 400,
	207, 400, /* RR */
	170, 400,
	133, 400,
	96, 400,
	59, 400,
	4, 400, /* Jail */
	4, 355,
	4, 318,
	4, 281,
	4, 243,
	4, 207, /* RR */
	4, 170,
	4, 133,
	4, 96,
	4, 59,
	4, 4, /* Free Parking */
	59, 4,
	96, 4,
	133, 4,
	170, 4,
	207, 4, /* RR */
	244, 4,
	281, 4,
	318, 4,
	355, 4,
	391, 4, /* Go to jail */
	409, 59,
	409, 96,
	409, 133,
	409, 170,
	409, 207, /* RR */
	409, 243,
	409, 281,
	409, 318,
	409, 355 
};

void update_board(void) 
{
	player *step;
	int first;
	int x, y, i, j, xstep, ystep;
	extern estate *estates[];

	xstep = ystep = 0;
	
	EENTER;

	gdk_draw_pixmap( board->window,
			 board->style->fg_gc[board->state],
			 boardpix, 0, 0, 0, 0,
			 450, 450 );

	for(i = 0; i < 40; i++) {
		if(estates[i]->ownable) {
			if(estates[i]->owner) {
				gdk_draw_pixmap( board->window,
						 board->style->fg_gc[board->state],
						 (estates[i]->mortgaged ? mflags : flags),
						 ((estates[i]->owner->playernum) * 16),
						 0,
						 base_coords[2*i] + 5,
						 base_coords[2*i + 1] + 10,
						 16, 24);
			} else {
				gdk_draw_pixmap( board->window,
						 board->style->fg_gc[board->state],
						 deeds, (estates[i]->group * 15), 0,
						 base_coords[2*i] + 5,
						 base_coords[2*i + 1] + 10,
						 16, 18);
			}
		}
		 
		if(estates[i]->houses) {
			if(i <= 9) {
				xstep = 9;
				ystep = 0;
			} else if (i <= 19) {
				xstep = 0;
				ystep = 9;
			} else if (i <= 29) {
				xstep = 9;
				ystep = 0;
			} else if (i <= 39) {
				xstep = 0;
				ystep = 9;
			}
			
			if(estates[i]->houses <= 4) {
				for(j = 0; j < estates[i]->houses; j++) {
					gdk_draw_pixmap( board->window,
							 board->style->fg_gc[board->state],
							 house,
							 0, 0,
							 base_house_coords[2*i] + (j * xstep),
							 base_house_coords[2*i + 1] + (j*ystep),
							 9, 10);
				}
			} else if (estates[i]->houses == 5) { /* hotel */
				gdk_draw_pixmap( board->window,
						 board->style->fg_gc[board->state],
						 house,
						 8, 0,
						 base_house_coords[2*i],
						 base_house_coords[2*i + 1],
						 12, 10);
			}
		}
	}
	
	if(players) {
		
		step = players;
		first = step->playernum;

		do {
			/* figure out where to put the icon */
			switch(step->position) {
			case 0:   /* go */
			case 10:  /* jail */
			case 20:  /* parking */
			case 30:  /* go to jail */
				if(step->playernum <= 3) {
					x = base_coords[2*(step->position)] + 10*(step->playernum);
					y = base_coords[2*(step->position) + 1] - 1;
				} else if(step->playernum <= 6) {
					x = base_coords[2*(step->position)] + 10*(step->playernum - 3);
					y = base_coords[2*(step->position) + 1] - 1 + 13;
				} else {
					x = base_coords[2*(step->position)] + 10*(step->playernum - 6);
					y = base_coords[2*(step->position) + 1] - 1 + 26;
				}
				
				
				break;
				
			case 1: case 2: case 3: case 4: case 5: case 6: case 7: case 8: case 9:
			case 11: case 21: case 22: case 23: case 24: case 25: case 26: case 27: case 28: case 29:
				/* top, bottom rows */
				if(step->playernum <= 3) {
					x = base_coords[2*(step->position)] + 10*(step->playernum);
					y = base_coords[2*(step->position) + 1] - 1;
				} else if(step->playernum <= 5) {
					x = base_coords[2*(step->position)] + 10*(step->playernum - 3);
					y = base_coords[2*(step->position) + 1] - 1 + 13;
				} else {
					x = base_coords[2*(step->position)] + 10*(step->playernum - 6);
				y = base_coords[2*(step->position) + 1] - 1 + 26;
				}
				break;
				
			default: /* left, right rows */
				if(step->playernum <= 4) {
					x = base_coords[2*(step->position)] + 10*(step->playernum);
					y = base_coords[2*(step->position) + 1] - 1;
				} else {
					x = base_coords[2*(step->position)] + 10*(step->playernum - 4);
				y = base_coords[2*(step->position) + 1] - 1 + 13;
				
				}
			}
			
			gdk_draw_pixmap( board->window,
					 board->style->fg_gc[board->state],
					 numbers,
					 (gint)(10 * (step->playernum)), 0,
					 x, y, 11, 13);
			step = step->next;
		} while( step->playernum != first );
	}
	
	EEXIT;
}
		
void update_statusbar(void) 
{
	extern gmonopprefs *prefs;
	player *me = player_of_id(prefs->me);
	if(me)
		statprintf("Cash: $%d", me->cash);
}

int init_board(void) 
{
	extern GtkWidget *mainwin;
	GtkStyle *style;
	char path[1024];
	char file[1024];
	
	ENTER;
	
	board = gtk_object_get_data(GTK_OBJECT(mainwin), "drawingarea");
	style = gtk_widget_get_style(board);

	strcpy(path, GTKMONOP_DATA);
	strcat(path, "/pixmaps");

	sprintf(file, "%s/mainboard.xpm", path);
	boardpix = gdk_pixmap_create_from_xpm(board->window, &bmap, NULL, file);
	sprintf(file, "%s/gtkmonop-numbers.xpm", path);
	numbers = gdk_pixmap_create_from_xpm(board->window, &nmap, NULL, file);
	sprintf(file, "%s/gtkmonop-flags.xpm", path);
	flags = gdk_pixmap_create_from_xpm(board->window, &fmap, NULL, file);
	sprintf(file, "%s/gtkmonop-mflags.xpm", path);
	mflags = gdk_pixmap_create_from_xpm(board->window, &mmap, NULL, file);
	sprintf(file, "%s/gtkmonop-houses.xpm", path);
	house = gdk_pixmap_create_from_xpm(board->window, &hmap, NULL, file);
	sprintf(file, "%s/gtkmonop-deeds.xpm", path);
	deeds = gdk_pixmap_create_from_xpm(board->window, &dmap, NULL, file);

	gtk_signal_connect(GTK_OBJECT(board),
			   "expose_event",
			   GTK_SIGNAL_FUNC(update_board),
			   (gpointer)(NULL));

	update_board();
	
	EXIT;
	return 1;
}

GtkWidget *pixmap_label (GdkPixmap *pm, GdkBitmap *mask, char *str)
{
	GtkWidget *box, *widget;
	box = gtk_hbox_new (FALSE, 5);
	widget = gtk_pixmap_new (pm, mask);
	gtk_widget_show (widget);
	gtk_box_pack_start (GTK_BOX(box), widget, TRUE, TRUE, 0);
	widget = gtk_label_new (str);
	gtk_widget_show (widget);
	gtk_box_pack_start (GTK_BOX(box), widget, TRUE, TRUE, 0);
	return box;
}

void init_colors (void)
{
	GtkWidget *label;
	GtkStyle  *rcstyle;

	/* create a dummy label so we can fetch the associated rc style */
	label = gtk_label_new ("");
	rcstyle = gtk_rc_get_style (label);
	gtk_widget_destroy (label);

	/* this is needed for some (broken?) themes */
	if (rcstyle == NULL)
	{
		rcstyle = gtk_style_new ();
	}
	else
	{
		gtk_style_ref (rcstyle);
	}
		
	/* make styles */
	active_text_style = gtk_style_copy (rcstyle);
	inactive_text_style = gtk_style_copy (rcstyle);

	/* load colours */
	active_text_style->fg[0] = rcstyle->fg[4];
	active_text_style->fg[0].red = 65535;

	/* free */
	gtk_style_unref (rcstyle);
}
