/* 
 * gtkmonop - the gtk+ monopoly simulator
 * Copyright (C) 2001 Erik Bourget
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/* xmlparse.c : functions for manipulating our gamestate via interpreting the
   xml */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/tree.h>

#include "misc_functions.h"
#include "gdk_functions.h"
#include "structs.h"
#include "assetbox.h"
#include "tradebox.h"
#include "auctionbox.h"
#include "stub.h"
#include "client.h"
#include "xmlparse.h"

void parse_auctionupdate (xmlDocPtr doc, xmlNodePtr cur) 
{
	extern GtkWidget *mainwin;
	extern auction *auctions;
	extern estate *estates[];
	char *buf, *buf2;
	auction *a, *astep;
	GtkWidget *notebook;
	GtkWidget *temp;
	char str[1024];

	notebook = gtk_object_get_data(GTK_OBJECT(mainwin), "notebook");
	if((buf = xmlGetProp(cur, (xmlChar *) "type"))) {
		if(!strcmp(buf, "new")) {
			/* alloc the auction */
			if(auctions) {
				a = (auction *) new_malloc(sizeof(auction));
				a->next = auctions->next;
				auctions->next = a;
			} else {
				a = auctions = (auction *) new_malloc(sizeof(auction));
			}
			
			a->next = (auction *)NULL;
			a->highbid = 0;
			a->highbidder = (player *)NULL;
			a->widget = NULL;
			a->count = 0;
			a->id = atoi(xmlGetProp(cur, (xmlChar *) "auctionid"));
			
			if(xmlGetProp(cur, (xmlChar *) "estateid"))
				sprintf(str, "Auction for %s", estates[atoi(xmlGetProp(cur, (xmlChar *) "estateid"))]->name);
			else
				sprintf(str, "Auction");
			
			a->tablabel = gtk_label_new(str);
			gtk_widget_show(a->tablabel);
			a->widget = new_auctionbox(a);
			gtk_widget_show(a->widget);
			
			gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
						 a->widget,
						 a->tablabel);
		} else if(!strcmp(buf, "edit")) {
			a = auction_of_id(atoi(xmlGetProp(cur, (xmlChar *) "auctionid")));
			if((buf2 = xmlGetProp(cur, (xmlChar *) "estateid"))) {
				a->estate = estates[atoi(buf2)];
				sprintf(str, "Auction for %s", a->estate->name);
				gtk_label_set_text(GTK_LABEL(a->tablabel), str);
			}
			if((buf2 = xmlGetProp(cur, (xmlChar *) "highbid"))) {
				update_auctionbox(a, 
						  player_of_id(atoi(xmlGetProp(cur, (xmlChar *) "highbidder"))),
						  atoi(buf2));
			}
			if((buf2 = xmlGetProp(cur, (xmlChar *) "message"))) {

				temp = gtk_object_get_data(GTK_OBJECT(a->widget), "label");
				if((xmlGetProp(cur, (xmlChar *) "status")) && atoi(xmlGetProp(cur, (xmlChar *) "status"))) {
					gtk_label_get(GTK_LABEL(temp), &buf);
					strcpy(str, buf);
					strcat(str, " ");
					strcat(str, buf2);
				} else
					strcpy(str, buf2);
				
				gtk_label_set_text(GTK_LABEL(temp), str);
			}
			
		} else if(!strcmp(buf, "completed")) {
			a = auction_of_id(atoi(xmlGetProp(cur, (xmlChar *) "auctionid")));
			kill_auctionbox(a);

			a = NULL;
			astep = auctions;
			while(astep->id != atoi(xmlGetProp(cur, (xmlChar *) "auctionid"))) {
				a = astep;
				astep = astep->next;
			}
			/* we will kill astep, a is the one before astep */

			if(a) a->next = astep->next; /* skip over astep */
			else auctions = astep->next; /* killing the first one */

			free(astep);
		}
	}
}

void parse_tradeupdate (xmlDocPtr doc, xmlNodePtr cur) 
{
	char *buf, *buf2;
	extern gmonopprefs *prefs;
	extern GtkWidget *mainwin;
	trade *t, *tstep;
	extern trade *trades;
	int i;
	char str[1024];
	xmlNodePtr step;
	GtkWidget *notebook;

	notebook = gtk_object_get_data(GTK_OBJECT(mainwin), "notebook");
	
	if((buf = xmlGetProp(cur, (xmlChar *) "type")))	{
		if(!strcmp(buf, "new")) {
			/* alloc the trade */
			if(trades) {
				t = (trade *) new_malloc(sizeof(trade));
				t->next = trades->next;
				trades->next = t;
			} else {
				t = trades = (trade *) new_malloc(sizeof(trade));
			}
			
			t->next = (trade *)NULL;
			t->name = (char *)NULL;
			t->id = atoi(xmlGetProp(cur, (xmlChar *) "tradeid"));
			for(i = 0; i < 40; i++)
				t->included[i] = 0;
			
			for(step = cur->children; step; step = step->next) {
				if(!strcmp(step->name, "player")) {
					if(atoi(xmlGetProp(step, (xmlChar *) "playerid")) == prefs->me)
						t->p[0] = player_of_id(prefs->me);
					else
						t->p[1] = player_of_id(atoi(xmlGetProp(step, (xmlChar *) "playerid")));
					/* add player to trade */
				}
			}
			
			sprintf(str, "Trade with %d. %s", t->p[1]->playernum, t->p[1]->name);
			t->tablabel = gtk_label_new(str);
			t->widget = new_tradebox(t);
			gtk_notebook_append_page(GTK_NOTEBOOK(notebook), 
						 t->widget,
						 t->tablabel);
		} else if(!strcmp(buf, "edit")) {
			t = trade_of_id(atoi(xmlGetProp(cur, (xmlChar *) "tradeid")));
			for(step = cur->children; step; step = step->next) {
				if(!strcmp(step->name, "estate")) {
					trades->included[atoi(xmlGetProp(cur, (xmlChar *) "tradeid"))] =
						atoi(xmlGetProp(step, (xmlChar *) "included"));
					update_tradebox(t,
							atoi(xmlGetProp(step, (xmlChar *) "estateid")), 
							atoi(xmlGetProp(cur, (xmlChar *) "actor")), 
							atoi(xmlGetProp(step, (xmlChar *) "included")));
				} else if(!strcmp(step->name, "player")) {
					if((buf2 = xmlGetProp(step, (xmlChar *) "accept"))) {
						i = ((atoi(xmlGetProp(step, (xmlChar *) "playerid"))) == prefs->me) ? 0 : 1;
						if(atoi(buf2))
							sprintf(str, "%s (Accepted)", t->p[i]->name);
						else
							sprintf(str, "%s", t->p[i]->name);
						
						gtk_frame_set_label(GTK_FRAME(t->frame[i]), str);
					}
					
					if((buf2 = xmlGetProp(step, (xmlChar *) "money"))) {
						i = ((atoi(xmlGetProp(step, (xmlChar *) "playerid"))) == prefs->me) ? 0 : 1;
						sprintf(str, "Money: $%d", atoi(buf2));
						gtk_ctree_node_set_text(GTK_CTREE(t->moneytree[i]), 
									gtk_object_get_data(GTK_OBJECT(t->moneytree[i]), "node"),
									0, str);
					}
				}
			}
		} else if(!strcmp(buf, "rejected") || !strcmp(buf, "completed")) {
			kill_tradebox(trade_of_id(atoi(xmlGetProp(cur, (xmlChar *) "tradeid"))));

			/* remove it from the linked list */

			t = NULL;
			tstep = trades;
			while(tstep->id != atoi(xmlGetProp(cur, (xmlChar *) "tradeid"))) {
				t = tstep;
				tstep = tstep->next;
			}
			/* we will kill tstep, t is the one before tstep */

			if(t) t->next = tstep->next; /* skip over tstep */
			else trades = tstep->next; /* killing the first one */

			free(tstep);
			return;
		}
	}
}

void parse_client(xmlDocPtr doc, xmlNodePtr cur) 
{
	/* prefs->me is my current playernum */
	char *buf;
	extern gmonopprefs *prefs;
		
	if((buf = xmlGetProp(cur, (xmlChar *) "playerid")))
		prefs->me = atoi(buf);
}

void parse_updategamelist(xmlDocPtr doc, xmlNodePtr cur) 
{
	extern gamelist *gameslist;
	game *trailer, *step;
	char *type;

	if(gameslist == (gamelist *)NULL) {
		gameslist = new_malloc(sizeof(gamelist));
		gameslist->games = (game *)NULL;
	}
	
	trailer = step = gameslist->games;

	type = xmlGetProp(cur, (xmlChar *) "type");
	
	if(!strcmp(type, "full")) {
		parse_gamelist(doc, cur);
		return;
	} else if(!strcmp(type, "add")) {
		
		while(step != (game *)NULL) {
			trailer = step;
			step = step->next;
		} /* find the last game in the list */
		
		for(cur = cur->children; cur; cur = cur->next) { /* for each game to add */
			if(!xmlStrcmp(cur->name, (const xmlChar *) "game")) {
				if(trailer == (game *)NULL) /* no games */
					step = gameslist->games = new_malloc(sizeof(game));
				else
					step = trailer->next = new_malloc(sizeof(game));
				step->id = xmlGetProp(cur, (const xmlChar *) "id");
				step->name = new_malloc(1024);
				if(xmlGetProp(cur, (const xmlChar *) "description"))
					strcpy(step->name, xmlGetProp(cur, (const xmlChar *) "description"));
				else
					strcpy(step->name, "");
				step->players = xmlGetProp(cur, (const xmlChar *) "players");
				step->next = NULL;
				trailer = step;
			}
		}
		return;
	} else if(!strcmp(type, "del")) {
		for(cur = cur->children; cur; cur = cur->next) { /* for each game to delete */
			for(step = gameslist->games; step; step = step ? step->next : NULL) { 
				/* for each game in our list */
				if(!xmlStrcmp(cur->name, (const xmlChar *) "game")) { /* if it's a game */
					if(!xmlStrcmp(xmlGetProp(cur, (xmlChar *) "id"), step->id)) { 
						/* and it matches the game in our list */
						if(trailer == step) { /* first one */
							gameslist->games = step->next;
							free(step); /* die! */
							step = gameslist->games;
						} else {
							trailer->next = step->next;
							free(step);
							step = trailer->next;
						} /* take it out of the list */
					}
				}
			}
		}
	} else if (!strcmp(type, "edit")) {
		for(cur = cur->children; cur; cur = cur->next) {
			/* for each game the server sends us */
			for(step = gameslist->games; step; step = step ? step->next : NULL) {
				/* for each game in our list */
				if(!strcmp(cur->name, (const xmlChar *) "game")) {
					if(!strcmp(xmlGetProp(cur, (xmlChar *) "id"), step->id)) {
						if(xmlGetProp(cur, (xmlChar *) "description"))
							step->name = xmlGetProp(cur, (xmlChar *) "description");
						if(xmlGetProp(cur, (xmlChar *) "players")) 
							step->players = xmlGetProp(cur, (xmlChar *) "players");
					}
				}
			}
		}
	}
}

/* that was painful. */	

void parse_gamelist(xmlDocPtr doc, xmlNodePtr cur) 
{
	extern gamelist *gameslist;
	game *trailer, *step;
	
	if(gameslist == (gamelist *)NULL) {
		gameslist = new_malloc(sizeof(gamelist));
		gameslist->games = (game *)NULL;
	}
	
	cur = cur->children;
	gameslist->games = new_malloc(sizeof(game));
	trailer = step = gameslist->games;
		
	if(cur == NULL) {
		free(gameslist->games);
		gameslist->games = (game *)NULL;
		return;
	}
	
	while(cur != NULL) {
		if (!xmlStrcmp(cur->name, (const xmlChar *) "game")) {
			step->id = xmlGetProp(cur, (const xmlChar *) "id");
			//			step->name = new_malloc(1024);
			if(xmlGetProp(cur, (const xmlChar *) "description"))
				step->name = xmlGetProp(cur, (const xmlChar *) "description");
			
			//				strcpy(step->name, xmlGetProp(cur, (const xmlChar *) "description"));
			step->players = xmlGetProp(cur, (const xmlChar *) "players");
		}
		step->next = new_malloc(sizeof(game));
		trailer = step;
		step = step->next;
		step->next = (game *)NULL;
		cur = cur->next;
	}
	
	if(trailer != step)
		free(trailer->next);
	
	trailer->next = (game *)NULL;
	return;
}

void parse_updateplayerlist(xmlDocPtr doc, xmlNodePtr cur) 
{
	extern player *players;
	extern GtkWidget *mainwin;
	player *step, *trailer;
	GtkWidget *playerlist;
	char *buf1, *buf2, *buf3, *bufx[3];

	int row;
	
	playerlist = (GtkWidget *)gtk_object_get_data(GTK_OBJECT(mainwin), "playerlist");
	gtk_clist_freeze(GTK_CLIST(playerlist));

	buf1 = buf2 = buf3 = malloc(20);
	strcpy(buf1, "0");

	if(!strcmp(xmlGetProp(cur, (const xmlChar *) "type"), "add")) {
		cur = cur->children;
		while(cur != NULL) { /* for each player */
			step = new_malloc(sizeof(player));
			
			if(!strcmp(cur->name, (const xmlChar *) "player")) {
				buf3 = step->name = xmlGetProp(cur, (const xmlChar *) "name");
				if(xmlGetProp(cur, (const xmlChar *) "playerid")) 
					buf2 = xmlGetProp(cur, (const xmlChar *) "playerid");
				else {
					buf2 = new_malloc(2);
					strcpy(buf2, "1");
				}
				
				step->playernum = atoi(buf2);
				step->cash = 0;
				step->position = 0;
				bufx[0] = buf1;
				bufx[1] = buf2;
				bufx[2] = buf3;
				/* add the row and make it unselectable */
				row = gtk_clist_append(GTK_CLIST(playerlist), bufx);
				gtk_clist_set_row_data(GTK_CLIST(playerlist), row, buf2);				
				
			}
			if(players) {
				trailer = players->next;
				players->next = step;
				step->next = trailer;
			} else {
				players = step;
				trailer = (player *) NULL;
				step->next = players;
			}
			cur = cur->next;
		}
	} else if(!strcmp(xmlGetProp(cur, (const xmlChar *) "type"), "full")) { 
		/* full */
		gtk_clist_clear(GTK_CLIST(playerlist));
		
		cur = cur->children;
		trailer = step = players = malloc(sizeof(player));
		
		while(cur != NULL) {
			if(!strcmp(cur->name, (const xmlChar *) "player")) {
				buf3 = step->name = xmlGetProp(cur, (const xmlChar *) "name");
				if(xmlGetProp(cur, (const xmlChar *) "playerid"))
					buf2 = xmlGetProp(cur, (const xmlChar *) "playerid");
				else {
					buf2 = new_malloc(2);
					strcpy(buf2, "0");
				}
				
				step->playernum = atoi(buf2);
				
				step->cash = 0;
				step->position = 0;
				bufx[0] = buf1;
				bufx[1] = buf2;
				bufx[2] = buf3;
				
				row = gtk_clist_append(GTK_CLIST(playerlist), bufx);
				gtk_clist_set_row_data(GTK_CLIST(playerlist), row, buf2);
				
			}
			step->next = new_malloc(sizeof(player));
			trailer = step;
			step = step->next;
			step->next = (player *)NULL;
			cur = cur->next;
		}
		
		free(trailer->next); // aka step
		trailer->next = players; // circular linked list
	}
	gtk_clist_thaw(GTK_CLIST(playerlist));
		

}

player *parse_playerlist(xmlDocPtr doc, xmlNodePtr cur) 
{
	player *ret, *step, *trailer;
	char *buf;
	extern GtkWidget *mainwin;
	GtkWidget *playerlist;
	playerlist = gtk_object_get_data(GTK_OBJECT(mainwin), "playerlist");

	gtk_clist_clear(GTK_CLIST(playerlist));
	gtk_clist_freeze(GTK_CLIST(playerlist));
	
	cur = cur->children;
	trailer = step = ret= malloc(sizeof(player));
	
	while(cur != NULL) {
		if(!xmlStrcmp(cur->name, (const xmlChar *) "player")) {
			buf = step->name = xmlGetProp(cur, (const xmlChar *) "name");
			step->playernum = atoi(xmlGetProp(cur, (const xmlChar *) "id"));
			step->cash = 0;
			step->position = 0;
			gtk_clist_append(GTK_CLIST(playerlist), &buf);
		}
		step->next = new_malloc(sizeof(player));
		trailer = step;
		step = step->next;
		step->next = (player *)NULL;
		cur = cur->next;
	}
	
	gtk_clist_thaw(GTK_CLIST(playerlist));

	free(trailer->next); // aka step
	trailer->next = ret; // circular linked list

	return ret;
}

void parse_joinedgame(xmlDocPtr doc, xmlNodePtr cur) 
{
	// return atoi(xmlGetProp(cur, (const xmlChar *) "playerid"));
}

void parse_leftgame(xmlDocPtr doc, xmlNodePtr cur) 
{
	player *left;
	player *step;
	extern player *players;
	extern estate *estates[];
	
	int i;

	left = player_of_id(atoi(xmlGetProp(cur, (const xmlChar *) "playerid")));
	step = left;
	
	
	if(left) {
		if(step->next != left) {
			
			while(step->next != left)
				step = step->next;
			
			step->next = left->next; /* skip over left */
			
			/* give property back */
			for(i = 0; i < 40; i++) {
				if(estates[i]->owner == left)
					estates[i]->owner = (player *)NULL;
			}
			
			if(players == left)
				players = left->next;
			
			free(left->name);
			free(left);
		} else {
			/* uh, that was us we just erased */
		}
	} /* otherwise our work is done */
}

void parse_msg(xmlDocPtr doc, xmlNodePtr cur) 
{
	extern int gamestarted;
	
	// extern GtkWidget *mainwin;
	//	GtkWidget *label = lookup_widget(GTK_WIDGET(mainwin), "actionlabel");
	
	if(!xmlStrcmp(xmlGetProp(cur, (const xmlChar *) "type"), "chat")) {
		chatprintf("<%s> %s\n", 
			   xmlGetProp(cur, (const xmlChar *) "author"),
			   xmlGetProp(cur, (const xmlChar *) "value"));
	} else if(!xmlStrcmp(xmlGetProp(cur, (const xmlChar *) "type"), "startgame")) {
		gamestarted = 1;
	} else {
		//gtk_label_set_text(GTK_LABEL(label), xmlGetProp(cur, (const xmlChar *) "value"));
		tprintf("%s\n", xmlGetProp(cur, (const xmlChar *) "value"));
	}
}

void parse_playerupdate(xmlDocPtr doc, xmlNodePtr cur) 
{
	extern player *players;
	player *player;
	char *str;

	int pid = atoi(xmlGetProp(cur, (const xmlChar *) "playerid"));
	
	if(pid == -1)
		return;
	
	player = player_of_id(pid);

	if(player) { /* not the first */
		if((str = xmlGetProp(cur, (const xmlChar *) "name")))
			player->name = str;
		if((str = xmlGetProp(cur, (const xmlChar *) "money"))) {
			player->cash = atoi(str);
		}
		
		if((str = xmlGetProp(cur, (const xmlChar *) "location"))) {
			/*
			// this is all for token movement stuff
			if(!xmlStrcmp(xmlGetProp(cur, (const xmlChar *) "directmove"), "0")) {
			//if it's not a direct move
			for(i = atoi(str); player->position != i; player->position++) {
			if(player->position >= 40) 
			//have to take an extra because of the increment 
			player->position -= 41;
			update_board();
			}
			} else {
			*/
			player->position = atoi(str);
			//}
			client_outmessage(OUT_CONFIRM, str);
		}
	} else { /* first player we're dealing with */
		players = malloc(sizeof(player));
		players->position = 0;
		players->cash = 0;

		players->playernum = atoi(xmlGetProp(cur, (const xmlChar *) "playerid"));
		if((str = xmlGetProp(cur, (const xmlChar *) "name")))
			players->name = str;
		if((str = xmlGetProp(cur, (const xmlChar *) "money")))
			players->cash = atoi(str);
		if((str = xmlGetProp(cur, (const xmlChar *) "location")))
			players->position = atoi(str);
		players->next = players;
	}
}


char *parse_movetoken(xmlDocPtr doc, xmlNodePtr cur) 
{
	int id;
	player *player;
	id = atoi(xmlGetProp(cur, (const xmlChar *) "player"));
	player = player_of_id(id);
	player->position = atoi(xmlGetProp(cur, (const xmlChar *) "location"));
	return xmlGetProp(cur, (const xmlChar *) "location");
}

void parse_newturn(xmlDocPtr doc, xmlNodePtr cur) 
{
	/* highlight the player whose turn it is and make everything else
	   unselectable */

	extern GtkWidget *mainwin;
	extern int gamestarted;
	int i, j, pid;
	char *buf;
	GtkWidget *playerlist = gtk_object_get_data(GTK_OBJECT(mainwin), "playerlist");

	gamestarted = 1;
	
	pid = atoi(xmlGetProp(cur, (const xmlChar *) "player"));
	j = 1921; /* not a player id */
	
	for(i = 0; j != pid; i++) {
		if(!gtk_clist_get_text(GTK_CLIST(playerlist), i, 1, &buf))
			return;
		j = atoi(buf);
	}
	for(j = 0; gtk_clist_get_text(GTK_CLIST(playerlist), j, 1, &buf); j++);
	
	if(i == 1) {
		/* remove currentturn icon from this guy */
		// gtk_clist_set_selectable(GTK_CLIST(playerlist), j-1, FALSE);
		
	} else {
		/* remove currentturn icon from this guy */
		//gtk_clist_set_selectable(GTK_CLIST(playerlist), i-2, FALSE);
	}

	/* add currentturn icon to this guy */
	//	gtk_clist_set_selectable(GTK_CLIST(playerlist), i-1, TRUE);
	gtk_clist_select_row(GTK_CLIST(playerlist), i-1, 0);
}

void parse_estateupdate(xmlDocPtr doc, xmlNodePtr cur) 
{
	/* fill in the estate information */
	extern estate *estates[];
	char *buf1, *buf2;

	if ((buf1 = xmlGetProp(cur, (const xmlChar *) "estateid"))) {
		estates[atoi(buf1)]->id = atoi(buf1);
		if((buf2 = xmlGetProp(cur, (const xmlChar *) "owner")) &&
		   (atoi(buf2) >= 0)) {
			update_assetbox(atoi(buf1), 
					estates[atoi(buf1)]->owner, 
					player_of_id(atoi(buf2)));
			estates[atoi(buf1)]->owner = player_of_id(atoi(buf2));
		}
		if((buf2 = xmlGetProp(cur, (const xmlChar *) "houses")))
			estates[atoi(buf1)]->houses = atoi(buf2);
		if((buf2 = xmlGetProp(cur, (const xmlChar *) "mortgaged")))
			estates[atoi(buf1)]->mortgaged = atoi(buf2);
		if((buf2 = xmlGetProp(cur, (const xmlChar *) "name"))) {
			estates[atoi(buf1)]->name = malloc(100);
			strcpy(estates[atoi(buf1)]->name, buf2);
		}
		if((buf2 = xmlGetProp(cur, (const xmlChar *) "groupid")))
			estates[atoi(buf1)]->group = atoi(buf2);
		if((buf2 = xmlGetProp(cur, (const xmlChar *) "can_be_mortgaged")))
			estates[atoi(buf1)]->mortgageable = atoi(buf2);
		if((buf2 = xmlGetProp(cur, (const xmlChar *) "can_be_unmortgaged")))
			estates[atoi(buf1)]->unmortgageable = atoi(buf2);
		if((buf2 = xmlGetProp(cur, (const xmlChar *) "can_buy_houses")))
			estates[atoi(buf1)]->canbuyhouses = atoi(buf2);
		if((buf2 = xmlGetProp(cur, (const xmlChar *) "can_sell_houses")))
			estates[atoi(buf1)]->cansellhouses = atoi(buf2);
		if((buf2 = xmlGetProp(cur, (const xmlChar *) "can_be_owned")))
			estates[atoi(buf1)]->ownable = atoi(buf2);
		if((buf2 = xmlGetProp(cur, (const xmlChar *) "houseprice")))
			estates[atoi(buf1)]->houseprice = atoi(buf2);
		update_board();
	}
}
